/****************************************************************************
* (C) 1999-2005 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Germany
*  
*  http://www.beck-ipc.com
*
* ---------------------------------------------------------------------------
* Module      : CGIAPI.C
* Function    : cgi api functions
* ---------------------------------------------------------------------------

$Header: CGIAPI.C, 19, 06.09.2006 11:21:11, Andre Pribil$

*****************************************************************************/

#include <STDLIB.H>
#include <DOS.H>

#include "config.h"
#include "CGIAPI.H"

/*************************************************************************/

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 CGI API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces CGIAPI\.H><efont> |

 @libcat1  CGI | cgi01 | CGI_Install | Install a CGI function |
 @libcat1  CGI | cgi02 | CGI_Delete | Remove a CGI function |

 @libcat1  CGI | cgi03 | CGI_SetMainPage | Select a new main page |
 @libcat1  CGI | cgi06 | CGI_GetMainPage | Get main page name |

 @libcat1  CGI | cgi04 | CGI_SetRootDir | Set Web server's root directory |
 @libcat1  CGI | cgi05 | CGI_GetRootDir | Get Web server's root directory |

 @libcat1  CGI | cgi07 | CGI_GetFormItem | Split a form item into name and value |
 @libcat1  CGI | cgi08 | CGI_GetNextItem | Return pointer to the next form item |
 
 @libcat1  CGI | cgi0A | CGI_Install_Func_OnErr | Install error cgi function |

 @libcat1  CGI | cgi0B | CGI_Install_MIME_Table | Install a custom MIME table |
 @libcat1  CGI | cgi0C | CGI_Remove_MIME_Table  | Remove the custom MIME table |

 @libcat2  SSI | cgi81 | SSI_Set_File_Filter | Set a SSI file filter |
 @libcat2  SSI | cgi82 | SSI_Register_Tag    | Register a SSI tag    |
 @libcat2  SSI | cgi83 | SSI_Set_TagLength   | Set the maximum SSI tag length |
 @libcat2  SSI | cgi84 | SSI_Send_Buffer     | Send a buffer over a HTTP connection |
*/

#define CGIVECT       0xAB
/*************************************************************************/
//install cgi function
/*************************************************************************/
#ifdef _cgi01_
/*
 @doc DOC_cgi01_
 ---------------------------------------------------------------
 Install a CGI function
 ---------------------------------------------------------------
 @clib  CGI_Install | Install a CGI function. |
        <cfont>int CGI_Install ( CGI_Entry far  *cgiptr );<efont>  |
 @parm  int  | cgiptr  | Pointer to a temporary <code>CGI_Entry<codee>
        <linkto>../../cgistruc.htm#CGI_Entry<linkmid>type<linktail> structure.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_NO_FREE_ENTRY or CGI_INVALID_METHOD
 @comm  This API function makes a copy of the information in the
        provided <code>CGI_Entry<codee> structure, so your structure
        at <code>cgiptr<codee> need not be persistent.
 @Also  cgi02.htm | CGI_Delete  | | | |
 @related <cfont>cgistat<codee> | ../../command.htm#cgistat | command | line
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x01<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_Install (CGI_Entry far  * cgiptr)
{
    _DX = FP_SEG(cgiptr);
    _SI = FP_OFF(cgiptr);

    _AH = CGI_INSTALL;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif

/*************************************************************************/
// delete cgi function
/*************************************************************************/
#ifdef _cgi02_
/*
 @doc DOC_cgi02_
 ---------------------------------------------------------------
 Remove a CGI function
 ---------------------------------------------------------------
 @clib  CGI_Delete | Remove a CGI function. |
        <cfont>int CGI_Delete ( char far * PathPtr );<efont>  |
 @parm  int  | PathPtr  | Pointer to the null terminated URL path name.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_NOT_FOUND
 @comm  The CGI function to be deleted is identified by the provided
        URL string.<nl>
        <nl>
        It is also possible to remove the two predefined
        cgi functions main.htm and ChipCfg from the table.<nl>
 @Also  cgi01.htm | CGI_Install  |
        cgi03.htm | CGI_SetMainPage | |
 @related <cfont>cgistat<codee> | ../../command.htm#cgistat | command | line
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x02<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_Delete (char far * PathPtr)
{
    _DX = FP_SEG(PathPtr);
    _SI = FP_OFF(PathPtr);

    _AH = CGI_REMOVE;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif


/*************************************************************************/
//Set main page
/*************************************************************************/
#ifdef _cgi03_
/*
 @doc DOC_cgi03_
 ---------------------------------------------------------------
 Set a new main page
 ---------------------------------------------------------------
 @clib  CGI_SetMainPage | Set a new main page. |
        <cfont>int CGI_SetMainPage ( const char far *mainPageName );<efont> |
 @parm  int  | mainPageName  | Pointer to name of new main page.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_INVALID_NAME
 @comm  The string at <code>mainPageName<codee> is null terminated
        with a maximum length of 64 characters (not counting the
        terminating zero).<nl>
 @Also  cgi06.htm | CGI_GetMainPage  |
        cgi04.htm | CGI_SetRootDir   | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x03<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_SetMainPage (const char far *mainPageName)
{
    _DX = FP_SEG(mainPageName);
    _SI = FP_OFF(mainPageName);

    _AH = CGI_SETMAIN;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif


/*************************************************************************/
//Set root dir
/*************************************************************************/
#ifdef _cgi04_
/*
 @doc DOC_cgi04_
 ---------------------------------------------------------------
 Set Web server's root directory
 ---------------------------------------------------------------
 @clib  CGI_SetRootDir | Set Web server's root directory. |
        <cfont>int CGI_SetRootDir ( const char far *rootDirName );<efont> |
 @parm  int  | rootDirName  | Pointer to the name of new root directory
        for Web server.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_INVALID_DIR
 @comm  The string at <code>rootDirName<codee> is null terminated with a maximum
        length of 64 characters (not counting the terminating zero).
 @Also  cgi05.htm | CGI_GetRootDir  |
        cgi03.htm | CGI_SetMainPage | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x04<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_SetRootDir (const char far *rootDirName)
{
    _DX = FP_SEG(rootDirName);
    _SI = FP_OFF(rootDirName);

    _AH = CGI_SETROOTDIR;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif


/*************************************************************************/
// Get root dir
/*************************************************************************/
#ifdef _cgi05_
/*
 @doc DOC_cgi05_
 ---------------------------------------------------------------
 Get Web server's root directory
 ---------------------------------------------------------------
 @clib  CGI_GetRootDir | Get Web server's root directory. |
        <cfont>int CGI_GetRootDir ( char far **rootDirName );<efont> |
 @parm  char far **  | rootDirName  | Contains pointer to root directory name.
 @rdesc 0:<tab>The variable rootDirName contains a pointer to the Web server's root directory name.
 @comm  The referenced root directory name string is null terminated and is in
        the  RTOS's data space.<nl>
        <nl>
        The caller must treat this string as a
        constant and must not modify it.
 @Also  cgi04.htm | CGI_SetRootDir  |
        cgi06.htm | CGI_GetMainPage | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x05<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_GetRootDir(char far **rootDirName)
{
    unsigned int tmp_es;

    _AH = CGI_GETROOTDIR;
    asm { int CGIVECT  } ;

    tmp_es = _ES;

    *rootDirName = (char far *)MK_FP( tmp_es, _DI );

    return _DX;
}
#endif



/*************************************************************************/
//Get main page name
/*************************************************************************/
#ifdef _cgi06_
/*
 @doc DOC_cgi06_
 ---------------------------------------------------------------
 Get main page name
 ---------------------------------------------------------------
 @clib  CGI_GetMainPage | Get main page name. |
        <cfont>int CGI_GetMainPage ( char far **mainPageName );<efont> |
 @parm  char far **  | mainPageName  | Contains pointer to current main page name.
 @rdesc 0:<tab>The variable mainPageName contains a pointer to a string containing name of current main page
 @comm  The referenced main page name string is null terminated and is in
        the  RTOS's data space.<nl>
        <nl>
        The caller must treat this string as a constant and must not modify it.
 @Also  cgi03.htm | CGI_SetMainPage |
        cgi05.htm | CGI_GetRootDir  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x06<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_GetMainPage(char far **mainPageName)
{
   unsigned int tmp_es;

   _AH = CGI_GETMAIN;
   asm { int CGIVECT  } ;

   tmp_es = _ES;

	*mainPageName = (char far *)MK_FP( tmp_es, _DI );

   return _DX;
}
#endif

/*************************************************************************/
// Get formitem
/*************************************************************************/
#ifdef _cgi07_
/*
 @doc DOC_cgi07_
 ---------------------------------------------------------------
 Split a form item into name and value
 ---------------------------------------------------------------
 @clib  CGI_GetFormItem | Split a form item into name and value. |
        <cfont>void CGI_GetFormItem ( const char far *ArgumentBufferPtr,<nl>
        <tab><tab><tab><tab><tab>&nbsp;&nbsp; FormItem far *formitem );<efont> |
 @parm  int  | ArgumentBufferPtr  | Input Parameter:<s2>
        Pointer to argument buffer to  be parsed.
 @parm  int  | formitem  | Input/Output Parameters:<s2>
        Pointer to a <code>FormItem<codee>
        <linkto>../../cgistruc.htm#typedef_FormItem<linkmid>type<linktail> structure
 @rdesc  -- none --
 @comm  On initial call, the <code>ArgumentBufferPtr<codee> provided by the caller
        is a copy of the <code>fArgumentBufferPtr<codee> member of
        the <code>rbCgi<codee>
        <linkto>../../cgistruc.htm#rpCgi<linkmid>structure<linktail> passed by the
        Web server to the CGI callback function.<s2> On subsequent calls
        to pick up additional form items, the pointer returned from the
        <linkto>cgi08.htm<linkmid>CGI_GetNextItem<linktail>
        function can be used here.<nl>
        <nl>
        Prior to calling the two members of the <code>FormItem<codee>
        <linkto>../../cgistruc.htm#typedef_FormItem<linkmid>structure<linktail>
        must be set by the caller.<s2>
        Both member pointers in the <code>FormItem<codee> data structure
        reference buffers allocated by the
        user, which will receive strings produced by this API call.<nl>
 @Also  cgi08.htm | CGI_GetNextItem  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x07<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void CGI_GetFormItem (const char far * ArgumentBufferPtr,
                      FormItem far * formitem)
{
    _BX = FP_SEG(ArgumentBufferPtr);
    _SI = FP_OFF(ArgumentBufferPtr);

    _ES = FP_SEG(formitem);
    _DI = FP_OFF(formitem);

    _AH = CGI_GETFORMITEM;
    asm { int CGIVECT  } ;
}
#endif

/*************************************************************************/
//Get next formitem
/*************************************************************************/
#ifdef _cgi08_
/*
 @doc DOC_cgi08_
 ---------------------------------------------------------------
 Return pointer to the next form item
 ---------------------------------------------------------------
 @clib  CGI_GetNextItem | Get the next form item. |
        <cfont>int CGI_GetNextItem ( char far *ArgumentBufferPtr,<nl>
        <tab><tab><tab><tab><tab>&nbsp;&nbsp;char far **nextItemString );<efont> |
 @parm  char far *  | ArgumentBufferPtr  | CGI request argument pointer.
 @parm  char far **  | nextItemString  | Pointer to the found item.
 @rdesc 0:<tab>Item was found<nl>
        -1:<tab>No more items found
 @comm  Most forms have more than one item.<s2> This function searches for
        the next form item in a CGI request argument string.<s2> It can
        only be used after a
        <linkto>cgi07.htm<linkmid>CGI_GetFormItem<linktail> call.<nl>
        <nl>
        The string referenced by <code>ArgumentBufferPtr<codee> is not
        modified in any way by this function.
 @Also  cgi07.htm | CGI_GetFormItem  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x08<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int CGI_GetNextItem (char far * ArgumentBufferPtr, char far **nextItemString)
{
    unsigned int tmp_es;

    _BX = FP_SEG(ArgumentBufferPtr);
    _SI = FP_OFF(ArgumentBufferPtr);

    _AH = CGI_FINDNEXTITEM;
    asm { int CGIVECT  } ;

    tmp_es = _ES;

    if (_DX == 0)
        *nextItemString = MK_FP( tmp_es, _DI );
    else
        *nextItemString = NULL;

    return _DX;
}
#endif


/*************************************************************************/
//Redirect to own files on http errors
/*************************************************************************/
#ifdef _cgi0A_
/*
 @doc DOC_cgi0A_
 ---------------------------------------------------------------
 Install CGI function on http error messages (e.g.404)
 ---------------------------------------------------------------
 @clib  CGI_Install_Func_OnErr | Install CGI function on http errors |
        <cfont>int CGI_Install_Func_OnErr( unsigned int error_state,
        <tab><tab><tab><tab><tab><tab>void far *CgiFuncOnErr);<nl>
 <efont> |
 @parm  unsigned int | error_state | http error state (see below)
 @parm  void far *  | CgiFuncOnErr   | CGI Function pointer
 @rdesc 0:<tab>CGI function installed<nl>
        -1:<tab>CGI function not installed <nl>
 @comm
 The CGI function must be implemented in the same way than a CGI function,
 installed with <linkto>../../cgi.htm#0xAB0x01<linkmid>CGI_INSTALL<linktail>.
 For deleting an entry, install a null pointer.<nl>
 @commtarget SC1x | Possible Http error states(parameter error_state):<nl>
 07: 401 Unauthorized<nl>
 08: 404 not found<nl>
 10: 500 Server error (File system error)<nl>
 22: 503 Service not available (Out of memory)<nl>
 @commtarget SC1x3 | Possible Http error states(parameter error_state):<nl>
 08: 404 not found<nl>

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x0A<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.11 | V1.01 | V1.01 | V0.90
*/
int CGI_Install_Func_OnErr( unsigned int error_state, void far *CgiFuncOnErr)
{
    asm les di,CgiFuncOnErr
    asm mov bx,error_state
    asm mov ah,CGI_ERR_FUNC;
    asm { int CGIVECT  } ;
    return _DX;
}
#endif

/*************************************************************************/
//Install user custom MIME table
/*************************************************************************/
#ifdef _cgi0B_
/*
 @doc DOC_cgi0B_
 ---------------------------------------------------------------
 Install user custom MIME table
 ---------------------------------------------------------------
 @clib  CGI_Install_MIME_Table | Install user custom MIME table |
        <cfont>int CGI_Install_MIME_Table( unsigned int entries,
        <tab><tab><tab><tab><tab><tab><tab>CUSTOM_MIMETYPES_TABLE far *mimeTable);<nl>
 <efont> |
 @parm  unsigned int | entries       | Number of entries in MIME table
 @parm  CUSTOM_MIMETYPES_TABLE far * | mimeTable | Pointer to custom
        <linkto>../../cgistruc.htm#typedef_CUSTOM_MIMETYPES_TABLE<linkmid>MIME table<linktail>
 @rdesc 0:<tab>OK<nl>
        -1:<tab>Error<nl>
 @comm The MIME table must be persistent. The RTOS does not make a copy.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x0B<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.20 | V1.20 | V1.20 | V1.05

 @target_clib V2.04
*/
int CGI_Install_MIME_Table( unsigned int entries,
                            CUSTOM_MIMETYPES_TABLE far *mimeTable)
{
    asm les di,mimeTable
    asm mov bx,entries
    asm mov ah,CGI_MIME_INSTALL;
    asm { int CGIVECT  } ;
    return _DX;
}
#endif


/*************************************************************************/
//Remove user custom MIME table
/*************************************************************************/
#ifdef _cgi0C_
/*
 @doc DOC_cgi0C_
 ---------------------------------------------------------------
 Remove user custom MIME table
 ---------------------------------------------------------------
 @clib  CGI_Remove_MIME_Table | Deinstall user custom MIME table |
        <cfont>int CGI_Remove_MIME_Table( void );<efont> |
 @rdesc 0:<tab>OK<nl>
        -1:<tab>Error<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x0C<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.20 | V1.20 | V1.20 | V1.05

 @target_clib V2.04
*/
int CGI_Remove_MIME_Table( void )
{
    asm mov ah,CGI_MIME_REMOVE;
    asm { int CGIVECT  } ;
    return _DX;
}
#endif

/*************************************************************************/
// SSI API
/*************************************************************************/
#ifdef _cgi81_
/*
 @doc DOC_cgi81_
 ---------------------------------------------------------------
 Set SSI file filter
 ---------------------------------------------------------------
 @clib  SSI_Set_File_Filter | Set Server Side Include (SSI) file filter |
        <cfont>int SSI_Set_File_Filter( char far * matchStr,<nl>
        <tab><tab><tab><tab><tab><tab> int matchType,<nl>
        <tab><tab><tab><tab><tab><tab> int server);<efont> |
 @parm  const char far * | matchStr  | Pointer to null-terminated string
        depending on the machted type
 @parm  int              | matchType | Match type, possible values are:<nl>
        <nl>
        <cfont>SSI_MATCH_FILE_NAME(0x01)<efont> = File name, Example: "table.htm"<nl>
        <cfont>SSI_MATCH_FILE_EXT (0x02)<efont> = File extension, Example: "htm"<nl>
        <cfont>SSI_MATCH_FILE_PATH(0x03)<efont> = Full path to file, Example: "/web/ssi/table.htm"<nl>
        <cfont>SSI_MATCH_DIR_NAME (0x04)<efont> = Directory name, Example: "ssi"<nl>
        <cfont>SSI_MATCH_DIR_PATH (0x05)<efont> = Directory path, Example: "/web/ssi"
 @parm  int              | server | 0 = register for HTTP server<nl>
                                    1 = register for HTTPS (SSL) server
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code CGI_INVALID_METHOD

 @comm  SSI commands are special HTML comment tags in web pages, and parsed
        by the web server while the pages are being served. They allow
        dynamically generated content to be inserted into a static page,
        without having to dynamically generate the hole page.<nl>
        There's no official standard for SSI command. Thus we provide an API
        to register a SSI tag string and a handler function. When a registered
        SSI tag is found in a page being served, the handler function will be
        called to process it.

 @Also  cgi82.htm | SSI_Register_Tag  |
        cgi83.htm | SSI_Set_TagLength |
                  |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x81<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V0.91

 @target_clib V2.02
*/
int SSI_Set_File_Filter( char far * matchStr,
                         int matchType,
                         int server)
{
    _DI = FP_OFF(matchStr);
    _ES = FP_SEG(matchStr);
    _BX = matchType;
    _CX = server;
    _AH = SSI_SET_FILE_FILTER;
    asm { int CGIVECT  } ;
    return _AX;
}
#endif


/*************************************************************************/
// SSI API
/*************************************************************************/
#ifdef _cgi82_
/*
 @doc DOC_cgi82_
 ---------------------------------------------------------------
 Register a SSI tag string
 ---------------------------------------------------------------
 @clib  SSI_Register_Tag | Register Server Side Include (SSI) tag string |
        <cfont>int SSI_Register_Tag( const char far * tagStr,<nl>
        <tab><tab><tab><tab><tab>&nbsp; SSI_Handler handler,<nl>
        <tab><tab><tab><tab><tab>&nbsp; int server);<efont> |
 @parm  const char far * | tagStr  | Tag string, Pointer to null-terminated string
 @parm  SSI_Handler      | handler | Pointer to callback function<nl>
        <nl>
        <bold>Callback function prototype:<bolde><nl>
        <cfont>typedef<nl>
        &nbsp; int huge (far *SSI_Handler) (unsigned long conhandle, const char far * tagPtr);<efont><nl>
        <nl>
        <bold>Possible callback function return values:<bolde><nl>
        <cfont>SSI_OUTPUT_ENABLE(0x00)<efont> = Enable web server to output static content<nl>
        <cfont>SSI_OUTPUT_DISABLE(0x01)<efont> = Disable web server to output static content<nl>
        <cfont>SSI_ABORT_PAGE(0x02)<efont> = Stop processing the current file<nl>
        other = The web server will send an error message and stop serving the HTTP request
 @parm  int              | server | 0 = register for HTTP server<nl>
                                    1 = register for HTTPS (SSL) server
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code CGI_INVALID_METHOD or
                   CGI_NO_FREE_ENTRY

 @Also  cgi81.htm | SSI_Set_File_Filter |
        cgi83.htm | SSI_Set_TagLength |
        cgi84.htm | SSI_Send_Buffer

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x82<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V0.91

 @target_clib V2.02
*/
int SSI_Register_Tag( const char far * tagStr,
                      SSI_Handler handler,
                      int server)
{
    _DI = FP_OFF(tagStr);
    _ES = FP_SEG(tagStr);
    _SI = FP_OFF(handler);
    _BX = FP_SEG(handler);
    _CX = server;
    _AH = SSI_REGISTER_TAG;
    asm { int CGIVECT  } ;
    return _AX;
}
#endif


/*************************************************************************/
// SSI API
/*************************************************************************/
#ifdef _cgi83_
/*
 @doc DOC_cgi83_
 ---------------------------------------------------------------
 Set maximum SSI tag length
 ---------------------------------------------------------------
 @clib  SSI_Set_TagLength | Set maximum length of Server Side Include (SSI) tag string |
        <cfont>int SSI_Set_TagLength( int tagLength );<efont> |
 @parm  int | tagLength | length in bytes (default is 90)
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code CGI_INVALID_METHOD

 @comm  Smaller values will optimize the performance of the SSI parser.

 @Also  cgi81.htm | SSI_Set_File_Filter |
        cgi83.htm | SSI_Set_TagLength |
                  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x83<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V0.91

 @target_clib V2.02
*/
int SSI_Set_TagLength( int tagLength )
{
    _BX = tagLength;
    _AH = SSI_SET_TAG_LEN;
    asm { int CGIVECT  } ;
    return _AX;
}
#endif

/*************************************************************************/
// SSI API
/*************************************************************************/
#ifdef _cgi84_
/*
 @doc DOC_cgi84_
 ---------------------------------------------------------------
 Transmit a user buffer
 ---------------------------------------------------------------
 @clib  SSI_Send_Buffer | Transmit a user buffer to the HTTP client through the HTTP connection |
        <cfont>int SSI_Send_Buffer( unsigned long conHandle,<nl>
        <tab><tab><tab><tab><tab> const char far *buffer,<nl>
        <tab><tab><tab><tab><tab> int bufferlen,<nl>
        <tab><tab><tab><tab><tab> int flag );<efont> |
 @parm  unsigned long   | conHandle | connection handle, given by callback function
 @parm  const char far *| buffer    | pointer to send buffer
 @parm  int | bufferlen | length of buffer in bytes, max. 32767
 @parm  int | flag      | flag, set to 1

 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code

 @comm  Smaller values will optimize the performance of the SSI parser.

 @Also  cgi81.htm | SSI_Set_File_Filter |
        cgi82.htm | SSI_Register_Tag    |
                  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x84<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V0.91

 @target_clib V2.02
*/
int SSI_Send_Buffer( unsigned long conHandle,
                     const char far *buffer,
                     int bufferlen,
                     int flag )
{
    _BX = (unsigned int)(conHandle>>16);
    _SI = (unsigned int)conHandle;
    _DI = FP_OFF(buffer);
    _ES = FP_SEG(buffer);
    _CX = bufferlen;
    _DX = flag;
    _AH = SSI_SEND_BUFFER;
    asm { int CGIVECT  } ;
    return _AX;
}
#endif

/*************************************************************************/
//end cgiapi.c